@@ -16,7 +16,6 @@ package net.minidev.json.parser;
  * limitations under the License.
  */
 import static net.minidev.json.parser.ParseException.ERROR_UNEXPECTED_CHAR;
-import static net.minidev.json.parser.ParseException.ERROR_UNEXPECTED_DUPLICATE_KEY;
 import static net.minidev.json.parser.ParseException.ERROR_UNEXPECTED_EOF;
 import static net.minidev.json.parser.ParseException.ERROR_UNEXPECTED_LEADING_0;
 import static net.minidev.json.parser.ParseException.ERROR_UNEXPECTED_TOKEN;
@@ -25,8 +24,9 @@ import static net.minidev.json.parser.ParseException.ERROR_UNEXPECTED_UNICODE;
 import java.io.IOException;
 import java.math.BigDecimal;
 import java.math.BigInteger;
-import java.util.List;
-import java.util.Map;
+
+import net.minidev.json.writer.JsonReader;
+import net.minidev.json.writer.JsonReaderI;
 
 /**
  * JSONParserBase is the common code between {@link JSONParserString} and
@@ -39,9 +39,10 @@ import java.util.Map;
  */
 abstract class JSONParserBase {
 	protected char c;
+	JsonReader base;
 	public final static byte EOI = 0x1A;
 	protected static final char MAX_STOP = 126; // '}' -> 125
-	//
+	private String lastKey;
 
 	protected static boolean[] stopAll = new boolean[MAX_STOP];
 	protected static boolean[] stopArray = new boolean[MAX_STOP];
@@ -61,8 +62,8 @@ abstract class JSONParserBase {
 	/*
 	 * End of static declaration
 	 */
-	protected ContainerFactory containerFactory;
-	protected ContentHandler handler;
+	//
+	//
 	protected final MSB sb = new MSB(15);
 	protected Object xo;
 	protected String xs;
@@ -134,31 +135,23 @@ abstract class JSONParserBase {
 	protected Number extractFloat() throws ParseException {
 		if (!acceptLeadinZero)
 			checkLeadinZero();
-		try {
-			if (!useHiPrecisionFloat)
-				return Float.parseFloat(xs);
-			if (xs.length() > 18) // follow JSonIJ parsing method
-				return new BigDecimal(xs);
-			return Double.parseDouble(xs);
-		} catch(NumberFormatException e) {
-			throw new ParseException(pos, ERROR_UNEXPECTED_TOKEN, xs);
-		}
+		if (!useHiPrecisionFloat)
+			return Float.parseFloat(xs);
+		if (xs.length() > 18) // follow JSonIJ parsing method
+			return new BigDecimal(xs);
+		return Double.parseDouble(xs);
 	}
 
 	/**
 	 * use to return Primitive Type, or String, Or JsonObject or JsonArray
 	 * generated by a ContainerFactory
 	 */
-	protected Object parse(ContainerFactory containerFactory, ContentHandler handler) throws ParseException {
-		this.containerFactory = containerFactory;
-		this.handler = handler;
-//		this.pos = -1;
-		Object result;
+	protected <T> T parse(JsonReaderI<T> mapper) throws ParseException {
+		this.pos = -1;
+		T result;
 		try {
 			read();
-			handler.startJSON();
-			result = readMain(stopX);
-			handler.endJSON();
+			result = readFirst(mapper);
 			if (checkTaillingData) {
 				if (!checkTaillingSpace)
 					skipSpace();
@@ -236,20 +229,15 @@ abstract class JSONParserBase {
 		return r;
 	}
 
-	/**
-	 * Read one char in this.c
-	 * 
-	 * @throws IOException
-	 */
 	abstract protected void read() throws IOException;
 
-	protected List<Object> readArray() throws ParseException, IOException {
-		List<Object> obj = containerFactory.createArrayContainer();
+	protected <T> T readArray(JsonReaderI<T> mapper) throws ParseException, IOException {
+		Object current = mapper.createArray();
 		if (c != '[')
 			throw new RuntimeException("Internal Error");
 		read();
 		boolean needData = false;
-		handler.startArray();
+		//
 		for (;;) {
 			switch (c) {
 			case ' ':
@@ -262,8 +250,8 @@ abstract class JSONParserBase {
 				if (needData && !acceptUselessComma)
 					throw new ParseException(pos, ERROR_UNEXPECTED_CHAR, (char) c);
 				read(); /* unstack */
-				handler.endArray();
-				return obj;
+				//
+				return mapper.convert(current);
 			case ':':
 			case '}':
 				throw new ParseException(pos, ERROR_UNEXPECTED_CHAR, (char) c);
@@ -276,7 +264,7 @@ abstract class JSONParserBase {
 			case EOI:
 				throw new ParseException(pos - 1, ERROR_UNEXPECTED_EOF, "EOF");
 			default:
-				obj.add(readMain(stopArray));
+				mapper.addValue(current, readMain(mapper, stopArray));
 				needData = false;
 				continue;
 			}
@@ -287,15 +275,7 @@ abstract class JSONParserBase {
 	 * use to return Primitive Type, or String, Or JsonObject or JsonArray
 	 * generated by a ContainerFactory
 	 */
-	// protected <T> T readFirst(AMapper<T> mapper) throws ParseException,
-	// IOException {
-	// }
-
-	/**
-	 * use to return Primitive Type, or String, Or JsonObject or JsonArray
-	 * generated by a ContainerFactory
-	 */
-	protected Object readMain(boolean stop[]) throws ParseException, IOException {
+	protected <T> T readFirst(JsonReaderI<T> mapper) throws ParseException, IOException {
 		for (;;) {
 			switch (c) {
 			// skip spaces
@@ -312,48 +292,150 @@ abstract class JSONParserBase {
 				throw new ParseException(pos, ERROR_UNEXPECTED_CHAR, c);
 				// start object
 			case '{':
-				return readObject();
+				return readObject(mapper);
 				// start Array
 			case '[':
-				return readArray();
+				return readArray(mapper);
 				// start string
 			case '"':
 			case '\'':
 				readString();
-				handler.primitive(xs);
+				//
+				return mapper.convert(xs);
+				// string or null
+			case 'n':
+				readNQString(stopX);
+				if ("null".equals(xs)) {
+					//
+					return null;
+				}
+				if (!acceptNonQuote)
+					throw new ParseException(pos, ERROR_UNEXPECTED_TOKEN, xs);
+				//
+				return mapper.convert(xs);
+				// string or false
+			case 'f':
+				readNQString(stopX);
+				if ("false".equals(xs)) {
+					//
+					return mapper.convert(Boolean.FALSE);
+				}
+				if (!acceptNonQuote)
+					throw new ParseException(pos, ERROR_UNEXPECTED_TOKEN, xs);
+				//
+				return mapper.convert(xs);
+				// string or true
+			case 't':
+				readNQString(stopX);
+				if ("true".equals(xs)) {
+					//
+					return mapper.convert(Boolean.TRUE);
+				}
+				if (!acceptNonQuote)
+					throw new ParseException(pos, ERROR_UNEXPECTED_TOKEN, xs);
+				//
+				return mapper.convert(xs);
+				// string or NaN
+			case 'N':
+				readNQString(stopX);
+				if (!acceptNaN)
+					throw new ParseException(pos, ERROR_UNEXPECTED_TOKEN, xs);
+				if ("NaN".equals(xs)) {
+					//
+					return mapper.convert(Float.valueOf(Float.NaN));
+				}
+				if (!acceptNonQuote)
+					throw new ParseException(pos, ERROR_UNEXPECTED_TOKEN, xs);
+				//
+				return mapper.convert(xs);
+				// digits
+			case '0':
+			case '1':
+			case '2':
+			case '3':
+			case '4':
+			case '5':
+			case '6':
+			case '7':
+			case '8':
+			case '9':
+			case '-':
+				xo = readNumber(stopX);
+				//
+				return mapper.convert(xo);
+			default:
+				readNQString(stopX);
+				if (!acceptNonQuote)
+					throw new ParseException(pos, ERROR_UNEXPECTED_TOKEN, xs);
+				//
+				return mapper.convert(xs);
+			}
+		}
+	}
+
+	/**
+	 * use to return Primitive Type, or String, Or JsonObject or JsonArray
+	 * generated by a ContainerFactory
+	 */
+	protected Object readMain(JsonReaderI<?> mapper, boolean stop[]) throws ParseException, IOException {
+		for (;;) {
+			switch (c) {
+			// skip spaces
+			case ' ':
+			case '\r':
+			case '\n':
+			case '\t':
+				read();
+				continue;
+				// invalid stats
+			case ':':
+			case '}':
+			case ']':
+				throw new ParseException(pos, ERROR_UNEXPECTED_CHAR, c);
+				// start object
+			case '{':
+				return readObject(mapper.startObject(lastKey));
+				// start Array
+			case '[':
+				return readArray(mapper.startArray(lastKey));
+				// start string
+			case '"':
+			case '\'':
+				readString();
+				//
 				return xs;
 				// string or null
 			case 'n':
 				readNQString(stop);
 				if ("null".equals(xs)) {
-					handler.primitive(null);
+					//
 					return null;
 				}
 				if (!acceptNonQuote)
 					throw new ParseException(pos, ERROR_UNEXPECTED_TOKEN, xs);
-				handler.primitive(xs);
+				//
 				return xs;
 				// string or false
 			case 'f':
 				readNQString(stop);
 				if ("false".equals(xs)) {
-					handler.primitive(Boolean.FALSE);
+					//
 					return Boolean.FALSE;
 				}
 				if (!acceptNonQuote)
 					throw new ParseException(pos, ERROR_UNEXPECTED_TOKEN, xs);
-				handler.primitive(xs);
+				//
 				return xs;
 				// string or true
 			case 't':
 				readNQString(stop);
 				if ("true".equals(xs)) {
-					handler.primitive(Boolean.TRUE);
+					//
 					return Boolean.TRUE;
 				}
 				if (!acceptNonQuote)
 					throw new ParseException(pos, ERROR_UNEXPECTED_TOKEN, xs);
-				handler.primitive(xs);
+				//
 				return xs;
 				// string or NaN
 			case 'N':
@@ -361,12 +443,12 @@ abstract class JSONParserBase {
 				if (!acceptNaN)
 					throw new ParseException(pos, ERROR_UNEXPECTED_TOKEN, xs);
 				if ("NaN".equals(xs)) {
-					handler.primitive(Float.NaN);
+					//
 					return Float.valueOf(Float.NaN);
 				}
 				if (!acceptNonQuote)
 					throw new ParseException(pos, ERROR_UNEXPECTED_TOKEN, xs);
-				handler.primitive(xs);
+				//
 				return xs;
 				// digits
 			case '0':
@@ -380,14 +462,14 @@ abstract class JSONParserBase {
 			case '8':
 			case '9':
 			case '-':
-				xo = readNumber(stop);
-				handler.primitive(xo);
-				return xo;
+				//
+				//
+				return readNumber(stop);
 			default:
 				readNQString(stop);
 				if (!acceptNonQuote)
 					throw new ParseException(pos, ERROR_UNEXPECTED_TOKEN, xs);
-				handler.primitive(xs);
+				//
 				return xs;
 			}
 		}
@@ -399,11 +481,11 @@ abstract class JSONParserBase {
 
 	abstract protected Object readNumber(boolean[] stop) throws ParseException, IOException;
 
-	protected Map<String, Object> readObject() throws ParseException, IOException {
-		Map<String, Object> obj = this.containerFactory.createObjectContainer();
+	protected <T> T readObject(JsonReaderI<T> mapper) throws ParseException, IOException {
+		//
 		if (c != '{')
 			throw new RuntimeException("Internal Error");
-		handler.startObject();
+		Object current = mapper.createObject();
 		boolean needData = false;
 		boolean acceptData = true;
 		for (;;) {
@@ -423,8 +505,8 @@ abstract class JSONParserBase {
 				if (needData && !acceptUselessComma)
 					throw new ParseException(pos, ERROR_UNEXPECTED_CHAR, (char) c);
 				read(); /* unstack */
-				handler.endObject();
-				return obj;
+				//
+				return mapper.convert(current);
 			case ',':
 				if (needData && !acceptUselessComma)
 					throw new ParseException(pos, ERROR_UNEXPECTED_CHAR, (char) c);
@@ -433,7 +515,7 @@ abstract class JSONParserBase {
 			case '"':
 			case '\'':
 			default:
-				int keyStart = pos;
+				// int keyStart = pos;
 				if (c == '\"' || c == '\'') {
 					readString();
 				} else {
@@ -444,31 +526,31 @@ abstract class JSONParserBase {
 				String key = xs;
 				if (!acceptData)
 					throw new ParseException(pos, ERROR_UNEXPECTED_TOKEN, key);
-				handler.startObjectEntry(key);
-				
-				//Skip spaces
+
+				// Skip spaces
 				skipSpace();
-				
+
 				if (c != ':') {
 					if (c == EOI)
 						throw new ParseException(pos - 1, ERROR_UNEXPECTED_EOF, null);
 					throw new ParseException(pos - 1, ERROR_UNEXPECTED_CHAR, c);
 				}
 				readNoEnd(); /* skip : */
-				Object duplicate = obj.put(key, readMain(stopValue));
-				if (duplicate != null)
-					throw new ParseException(keyStart, ERROR_UNEXPECTED_DUPLICATE_KEY, key);
-				handler.endObjectEntry();
+				lastKey = key;
+				Object value = readMain(mapper, stopValue);
+				mapper.setValue(current, key, value);
+				lastKey = null;
+
+				// Object duplicate = obj.put(key, readMain(stopValue));
+				// if (duplicate != null)
+				// throw new ParseException(keyStart, ERROR_UNEXPECTED_DUPLICATE_KEY, key);
+				// handler.endObjectEntry();
 				// should loop skipping read step
-				//
-				//
-				//
-				//
 				skipSpace();
 				if (c == '}') {
 					read(); /* unstack */
-					handler.endObject();
-					return obj;
+					//
+					return mapper.convert(current);
 				}
 				if (c == EOI) // Fixed on 18/10/2011 reported by vladimir
 					throw new ParseException(pos - 1, ERROR_UNEXPECTED_EOF, null);
@@ -477,7 +559,7 @@ abstract class JSONParserBase {
 					acceptData = needData = true;
 				else
 					throw new ParseException(pos - 1, ERROR_UNEXPECTED_TOKEN, c);
-				//	acceptData = needData = false;
+				// acceptData = needData = false;
 			}
 		}
 	}
@@ -539,8 +621,9 @@ abstract class JSONParserBase {
 				case 'u':
 					sb.append(readUnicode(4));
 					break;
-				case 'x': // issue 39
+				case 'x':
 					sb.append(readUnicode(2));
+					break;
 				default:
 					break;
 				}
