@@ -7,13 +7,26 @@ import net.minidev.json.parser.ParseException;
 public class MustThrows {
 
 	public static void testStrictInvalidJson(String json, int execptionType) throws Exception {
-		testInvalidJson(json, JSONParser.MODE_RFC4627, execptionType);
+		testStrictInvalidJson(json, execptionType, null);
+
 	}
-	
+
+	public static void testStrictInvalidJson(String json, int execptionType, Class<?> cls) throws Exception {
+		testInvalidJson(json, JSONParser.MODE_RFC4627, execptionType, cls);
+	}
+
 	public static void testInvalidJson(String json, int permissifMode, int execptionType) throws Exception {
+		testInvalidJson(json, permissifMode, execptionType, null);
+	}
+
+	public static void testInvalidJson(String json, int permissifMode, int execptionType, Class<?> cls)
+			throws Exception {
 		JSONParser p = new JSONParser(permissifMode);
 		try {
-			p.parse(json);
+			if (cls == null)
+				p.parse(json);
+			else
+				p.parse(json, cls);
 			TestCase.assertFalse("Exception Should Occure parsing:" + json, true);
 		} catch (ParseException e) {
 			if (execptionType == -1)
@@ -21,4 +34,5 @@ public class MustThrows {
 			TestCase.assertEquals(execptionType, e.getErrorType());
 		}
 	}
+
 }
