@@ -14,7 +14,7 @@ public class TestInvalidNumber extends TestCase {
 		String comp = JSONValue.toJSONString(o, JSONStyle.MAX_COMPRESS);
 		assertEquals("{a:\"51e88\"}", comp);
 
-		o = (JSONObject)JSONValue.parse(comp);
+		o = JSONValue.parse(comp, JSONObject.class);
 		assertEquals(o.get("a"), test);
 	}
 
@@ -25,7 +25,7 @@ public class TestInvalidNumber extends TestCase {
 		String comp = JSONValue.toJSONString(o, JSONStyle.MAX_COMPRESS);
 		assertEquals("{a:\"51e+88\"}", comp);
 
-		o = (JSONObject)JSONValue.parse(comp);
+		o = JSONValue.parse(comp, JSONObject.class);
 		assertEquals(o.get("a"), test);
 	}
 
@@ -36,7 +36,7 @@ public class TestInvalidNumber extends TestCase {
 		String comp = JSONValue.toJSONString(o, JSONStyle.MAX_COMPRESS);
 		assertEquals("{a:\"51e-88\"}", comp);
 
-		o = (JSONObject)JSONValue.parse(comp);
+		o = JSONValue.parse(comp, JSONObject.class);
 		assertEquals(o.get("a"), test);
 	}
 
@@ -47,7 +47,7 @@ public class TestInvalidNumber extends TestCase {
 		String comp = JSONValue.toJSONString(o, JSONStyle.MAX_COMPRESS);
 		assertEquals("{a:51ee88}", comp);
 		
-		o = (JSONObject)JSONValue.parse(comp);
+		o = JSONValue.parse(comp, JSONObject.class);
 		assertEquals(o.get("a"), test);
 	}
 
